package Cpanel::API::JetBackup;

use strict;
use CGI;

our $VERSION = '1.0';
 
use Cpanel                   ();
use Cpanel::API              ();
use Cpanel::Locale           ();
use Cpanel::Logger           ();
use Cpanel::LiveAPI          ();
use Data::Dumper             ();
use Cpanel::Wrap             ();
use CGI;
use URI::Escape;
use PHP::Serialization qw( unserialize );
use MIME::Base64;

sub _escape_hash {
	my %hash = @_;
	my @pairs;
	for my $key (keys %hash) {
		push @pairs, join "=", map { uri_escape($_) } $key, $hash{$key};
	}
	return join "&", @pairs;
}

# Globals
my $logger;
my $locale;

sub _request {
	my($args, $result) = @_;
	#my($function, $data) = $args->get('function', 'data');

	_initialize();

	#$args->add('language', $locale->get_language_tag());

	my $feature = 'jetbackup';

	if(!main::hasfeature($feature))
	{
		$result->error('_ERROR_FEATURE', $feature);
		return;
	}

	if($Cpanel::CPDATA{'DEMO'})
	{
		$result->error('_ERROR_DEMO_MODE', $feature);
		return;
	}

	my $wrap_result = Cpanel::Wrap::send_cpwrapd_request(
		'namespace' => "JetApps",
		'module'    => "JetBackup",
		'function'  => $args->get('apifunction'),
		'data'      => _escape_hash(%{ $args->{_args} }),
		#'action' 	=> 'fetch',
		'env'    	=> Cpanel::Wrap::Config::safe_hashref_of_allowed_env(),
	);

    my $error = $wrap_result->{'error'};
	my $data = $wrap_result->{'data'};

    if($error) {
		if($wrap_result->{'statusmsg'}) { $result->error($wrap_result->{'statusmsg'}) }
		else { $result->error('Error code ' . $wrap_result->{'exit_code'} . ' returned: ' . $wrap_result->{'data'}) }
		return 0;
	}
	elsif(ref($data) || defined($data)) {
		if(ref($data)) { my $data = Data::Dumper::Dumper($data); }

		my $response = unserialize(decode_base64($data));

		$result->data($response->{'data'});

		if($response->{'success'}) {
			$result->message($response->{'message'});
			return 1;
		} else {
			$result->error($response->{'message'});
			return 0;
		}
	}
	else {
		$result->error('Request Failed: ' . $wrap_result->{'statusmsg'});
		return 0;
	}
}

sub _initialize {
	$logger ||= Cpanel::Logger->new();
	$locale ||= Cpanel::Locale->get_handle();
	return 1;
}

sub getAccountDetails {
	my($args, $result) = @_;
	$args->add('apifunction', 'getAccountDetails');
	return _request($args, $result);
}

sub manageAccount {
	my($args, $result) = @_;
	$args->add('apifunction', 'manageAccount');
	return _request($args, $result);
}

sub getGDPR {
	my($args, $result) = @_;
	$args->add('apifunction', 'getGDPR');
	return _request($args, $result);
}

sub manageGDPR {
	my($args, $result) = @_;
	$args->add('apifunction', 'manageGDPR');
	my $request = new CGI;
	$args->add('ip', $request->remote_addr());
	return _request($args, $result);
}

sub fileManager {
	my($args, $result) = @_;
	$args->add('apifunction', 'fileManager');
	return _request($args, $result);
}

sub listBackups {
	my($args, $result) = @_;
	$args->add('apifunction', 'listBackups');
	return _request($args, $result);
}

sub manageBackup {
	my($args, $result) = @_;
	$args->add('apifunction', 'manageBackup');
	return _request($args, $result);
}

sub getConfigurations {
	my($args, $result) = @_;
	$args->add('apifunction', 'getConfigurations');
	return _request($args, $result);
}

sub getInfo {
	my($args, $result) = @_;
	$args->add('apifunction', 'getInfo');
	return _request($args, $result);
}

sub listQueueItems {
	my($args, $result) = @_;
	$args->add('apifunction', 'listQueueItems');
	return _request($args, $result);
}

sub listDownloads {
	my($args, $result) = @_;
	$args->add('apifunction', 'listDownloads');
	return _request($args, $result);
}

sub getBackupDownloads {
	my($args, $result) = @_;
	$args->add('apifunction', 'getBackupDownloads');
	return _request($args, $result);
}

sub deleteDownload {
	my($args, $result) = @_;
	$args->add('apifunction', 'deleteDownload');
	return _request($args, $result);
}

sub getDownload {
	my($args, $result) = @_;
	$args->add('apifunction', 'getDownload');
	return _request($args, $result);
}

sub addQueueSnapshot {
	my($args, $result) = @_;
	$args->add('apifunction', 'addQueueSnapshot');
	return _request($args, $result);
}

sub addQueueDownload {
	my($args, $result) = @_;
	$args->add('apifunction', 'addQueueDownload');
	return _request($args, $result);
}

sub addQueueRestore {
	my($args, $result) = @_;
	$args->add('apifunction', 'addQueueRestore');
	return _request($args, $result);
}

sub listPermissions {
	my($args, $result) = @_;
	$args->add('apifunction', 'listPermissions');
	return _request($args, $result);
}

1;
