'use strict';

define([
	'app',
	'libraries/permissions',
	'libraries/consts'
	], function(app, PERMISSIONS, CONSTS) {
	app.controller("accountFilterManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'AccountFilters');

				$scope.details = {
					name: ''
				};

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;
				$scope.range = "0123456789_abcdefghijklmnopqrstuvwxyz";

				$scope.conditions = [
					{ label: $scope.LANG.t("Include"), value: CONSTS.ACCOUNT_FILTER_CONDITION_INCLUDE },
					{ label: $scope.LANG.t("Exclude"), value: CONSTS.ACCOUNT_FILTER_CONDITION_EXCLUDE }
				];

				$scope.types = [
					{ label: $scope.LANG.t("Accounts Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_ACCOUNT, template: 'account' },
					{ label: $scope.LANG.t("Resellers Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_RESELLER, template: 'reseller' },
					{ label: $scope.LANG.t("Suspension Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_SUSPENSION, template: 'suspension' },
					{ label: $scope.LANG.t("Disk Space Usage Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_DISK_USAGE, template: 'diskusage' },
					{ label: $scope.LANG.t("Inodes Usage Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_INODE_USAGE, template: 'inodeusage' },
					{ label: $scope.LANG.t("Packages Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_PACKAGE, template: 'package' },
					{ label: $scope.LANG.t("Characters Range Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_RANGE, template: 'range' },
					{ label: $scope.LANG.t("Regular Expression Filter"), value: CONSTS.ACCOUNT_FILTER_TYPE_REGEX, template: 'regex' }
				];

				$scope.loadFilterType = function(type, defaults) {

					$scope.saveData = utils.duplicateObject($scope.details);

					if(defaults)
					{
						$scope.saveData.condition = CONSTS.ACCOUNT_FILTER_CONDITION_INCLUDE;
						$scope.saveData.type = type;

						switch(type)
						{
							case CONSTS.ACCOUNT_FILTER_TYPE_ACCOUNT:
							case CONSTS.ACCOUNT_FILTER_TYPE_RESELLER:
							case CONSTS.ACCOUNT_FILTER_TYPE_PACKAGE:
								$scope.saveData.list = [];
							break;

							case CONSTS.ACCOUNT_FILTER_TYPE_INODE_USAGE:
							case CONSTS.ACCOUNT_FILTER_TYPE_DISK_USAGE:
								$scope.saveData.usage = 0;
							break;

							case CONSTS.ACCOUNT_FILTER_TYPE_RANGE:
								$scope.saveData.rangestart = '0';
								$scope.saveData.rangeend = 'z';
							break;

							case CONSTS.ACCOUNT_FILTER_TYPE_REGEX:
								$scope.saveData.regex = '';
							break;
						}
					}

					if($scope.saveData.type == CONSTS.ACCOUNT_FILTER_TYPE_DISK_USAGE || $scope.saveData.type == CONSTS.ACCOUNT_FILTER_TYPE_INODE_USAGE)
					{
						$scope.conditions[0].label = $scope.LANG.t("Include Below (or equals)");
						$scope.conditions[1].label = $scope.LANG.t("Include Above (or equals)");
					}
					else
					{
						$scope.conditions[0].label = $scope.LANG.t("Include");
						$scope.conditions[1].label = $scope.LANG.t("Exclude");
					}

					for(var i = 0; i < $scope.types.length; i++) {
						if($scope.types[i].value === type) {
							$scope.filterType = 'templates/jetbackup/app/views/accountFilters/' + $scope.types[i].template + '.ptt';
							return;
						}
					}

				};


				$scope.itemsSelection = function () {

					var types = {};
					types[CONSTS.ACCOUNT_FILTER_TYPE_ACCOUNT] = { template: 'accountsSelection', controller: 'accountsSelection', resolve: { accounts: function() { return $scope.saveData.list; }, callParams: function () { return ''; } } };
					types[CONSTS.ACCOUNT_FILTER_TYPE_RESELLER] = { template: 'accountsSelection', controller: 'resellersSelection', resolve: { accounts: function() { return $scope.saveData.list; } } };
					types[CONSTS.ACCOUNT_FILTER_TYPE_PACKAGE] = { template: 'packagesSelection', controller: 'packagesSelection', resolve: { packages: function() { return $scope.saveData.list; } } };

					var modalInstance = $uibModal.open({
						animation: true,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/' + types[$scope.saveData.type].template + '.ptt',
						controller: types[$scope.saveData.type].controller,
						controllerAs: '$ctrl',
						resolve: types[$scope.saveData.type].resolve
					});

					modalInstance.result.then(function(selected) {
						$scope.saveData.list = selected;
					}, function () {
						//$log.info('Modal dismissed at: ' + new Date());
					});
				};

				$scope.searchAccounts = function(query) {

					if(query)
					{
						var reseller = $scope.saveData.type === CONSTS.ACCOUNT_FILTER_TYPE_RESELLER;
						var deferred = $q.defer();

						jetapi.listAccounts({ filter: query, sort: { username: 1 } }, function(response) {

							var results = [];
							for(var i = 0; i < response.data.accounts.length; i++) {

								var account = response.data.accounts[i];

								if(
									(!reseller && $scope.saveData.list.indexOf(account.username) >= 0)
									||
									(reseller && (!account.reseller || $scope.saveData.list.indexOf(account.username) >= 0))
								) continue;

								results.push(account);
							}
							deferred.resolve( results );
						});

						return deferred.promise;
					}

					return [];
				};

				$scope.searchPackages = function(query) {

					if(query)
					{
						var deferred = $q.defer();

						jetapi.listPackages({ filter: query, sort: { name: 1 } }, function(response) {

							var results = [];
							for(var i = 0; i < response.data.packages.length; i++) {

								var packagedetails = response.data.packages[i];
								if($scope.saveData.list.indexOf(packagedetails.name) >= 0) continue;
								results.push(packagedetails);
							}
							deferred.resolve( results );
						});

						return deferred.promise;
					}

					return [];
				};

				$scope.selectItem = function(item, obj) {
					if(item === undefined) return;
					$scope.saveData.list.push(item);
					obj.searchItemText = '';
				};

				$scope.rangeStartValueCheck = function (char) {
					return char <= $scope.saveData.rangeend;
				};

				$scope.rangeEndValueCheck = function (char) {
					return char >= $scope.saveData.rangestart;
				};

				$scope.fetchFilterData = function(filter_id) {
					jetapi.getAccountFilter({ _id: filter_id }, function(response) {
						$scope.details = response.data;

						if($scope.details.type == CONSTS.ACCOUNT_FILTER_TYPE_DISK_USAGE && $scope.details.usage > 0) {
							$scope.details.usage = (($scope.details.usage / 1024) / 1024);
						}

						$scope.loadFilterType($scope.details.type);
					});
				};

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('createFilter', function (event) {
					$scope.saveChanges(true, function(response) {
						$scope.$emit('filterResponse', response);
					});
				});

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled && !$scope.popup && !$scope.wizardData)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/accountFilters');
				};

				$scope.saveChanges = function(apply, callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details, [], function(key) {
						return ($scope.details._id === undefined && key === 'type');
					});

					apiParams.action = $scope.details._id ? 'modify' : 'create';

					if(apiParams.usage !== undefined && $scope.saveData.type == CONSTS.ACCOUNT_FILTER_TYPE_DISK_USAGE && apiParams.usage > 0) {
						apiParams.usage = ((apiParams.usage * 1024) * 1024);
					}

					jetapi.manageAccountFilter(apiParams, function(response) {

						$scope.saveing = false;

						if(callback !== undefined && typeof callback === 'function') {
							callback(response);
							return;
						}

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(apply) $location.path('/accountFilterManage/' + response.data._id);
							else $location.path('/accountFilters');

							SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
						}
						else
						{
							$scope.status = {
								message: response.message,
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				var filter_id = $routeParams.id;
				if(filter_id) $timeout($scope.fetchFilterData(filter_id));
				else $scope.loadFilterType(CONSTS.ACCOUNT_FILTER_TYPE_ACCOUNT, true);
	}]);
});