'use strict';

define([
	'app',
	'libraries/utils',
	'libraries/permissions',
	'libraries/consts'
], function(app, UTILS, PERMISSIONS, CONSTS) {
	app.controller("backupManagerDirectories",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {

				$scope.directories = [];
				$scope.directory_data = {};
				$scope.backups = {};
				$scope.downloads = {};
				$scope.directories_loaded = false;
				$scope.loadingDirectories = false;
				$scope.checked = false;
				$scope.status = undefined;

				meta = meta.new("backup_manager_directories");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["name","type"]);
				meta.setTotalItems($scope.directories.length);

				$scope.destinations = [{_id: '', name: $scope.LANG.t('All') }];

				$scope.flags = [
					{ label: $scope.LANG.t("All"), value: 0 },
					{ label: $scope.LANG.t("Incremental"), value: 1 },
					{ label: $scope.LANG.t("Archived"), value: 2 },
					{ label: $scope.LANG.t("Compressed"), value: 4 }
				];

				$scope.filter = {
					destination: '',
					structure: 0
				};

				var toggleLock = function(backups, lock, ttl, callback) {

					var ids = [];
					for(var i = 0; i < backups.length; i++) ids.push(backups[i]._id);
					jetapi.manageBackup({ _id: ids, lock_ttl: ttl, locked: lock }, function(response) {
						if(callback !== undefined) callback(response);
					});
				};

				var getCheckedAndToggleLock = function(state, ttl) {
					var checked = [];
					for (var directory in $scope.directory_data) {
						if ($scope.directory_data[directory].checked) {
							checked.push($scope.backups[directory][$scope.directory_data[directory].selected_id]);
						}
					}
					toggleLock(checked, state? state:0, ttl? ttl:0, function (response) {
						if(!response.success) {
							SweetAlert.error($scope.LANG.t("Error!"), response.message);
							return;
						}

						for(var i = 0; i < checked.length; i++) checked[i].locked = state ? state: 0;
						SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("The selected backups locked successfully"));
					});
				};

				$scope.lockSelected = function() {
					$scope.lockTimeSelection(function (ttl) {
						getCheckedAndToggleLock(1,ttl);
					});
				};

				$scope.unlockSelected = function() {
					getCheckedAndToggleLock(0,0);
				};

				$scope.toggleLock = function(backup) {
					var status = !backup.locked;
					$scope.lockTimeSelection(function (ttl) {
						toggleLock([backup], status, ttl, function (response) {

							if (response.success) {
								backup.locked = status;
								SweetAlert.success($scope.LANG.t("Success!"), response.message);
								return;
							}
							SweetAlert.error($scope.LANG.t("Error!"), response.message);
						});
					}, !status);
				};

				$scope.editNotes = function (backup) {
					if(!PERMISSIONS.canAddBackupNotes) return;
					backup.editing = true;
				};

				$scope.saveNotes = function (backup) {
					jetapi.manageBackup({ _id: backup._id, notes: backup.notes });
					backup.editing = false;
				};

				$scope.showDirectoryBackup = function(directory) {
					return $scope.directory_data[directory.identity] !== undefined && $scope.directory_data[directory.identity].selected_id !== undefined;
				};

				$scope.setCheckAll = function() {
					for(var i = 0; i < $scope.directories.length; i++) {
						if(!$scope.directory_data[$scope.directories[i].identity].restore && !$scope.directory_data[$scope.directories[i].identity].download) {
							$scope.directory_data[$scope.directories[i].identity].checked = $scope.checkall;
						}
					}
					$scope.checkChanged();
				};

				$scope.checkChanged = function () {
					for(var directory in $scope.directory_data) {
						if($scope.directory_data[directory].checked)
						{
							$scope.checked = true;
							return;
						}
					}

					$scope.checked = false;
				};

				$scope.checkDirectory = function (directory) {
					if($scope.directory_data[directory.identity].selected_id === undefined || $scope.directory_data[directory.identity].restore || $scope.directory_data[directory.identity].download) return;
					$scope.directory_data[directory.identity].checked = !$scope.directory_data[directory.identity].checked;
					$scope.checkChanged();
				};

				/*
				$scope.restoreSelected = function () {
					for(var directory in $scope.directory_data) {
						if($scope.directory_data[directory].checked)
						{
							$scope.selectedDirectory = directory;

							jetapi.addQueueRestore({ _id: $scope.directory_data[directory].selected_id }, function(response, params) {
								$scope.directory_data[params.directory].checked = false;

								if(response.success) {
									$scope.backups[params.directory][$scope.directory_data[params.directory].selected_id].queue.restore = response.data.queue_id;
									$scope.directory_data[params.directory].restore = response.data.queue_id;
								} else {

									failed.push(i18next.t('Directory "%s"', params.directory) + ": " + response.message);

									$scope.status = {
										message: failed.join("<br />"),
										type: "danger",
										closeable: false,
										ttl: 10000
									};
								}
							}, {directory: directory});
						}
					}

					$scope.checkall = false;
					$scope.checked = false;
					SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("The selected directories added to the restore queue successfully"));
				};
				*/

				$scope.downloadSelected = function () {

					$scope.status = undefined;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("The selected backups will be added to the download queue."),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							var failed = ["<br />"];

							for(var directory in $scope.directory_data) {
								if($scope.directory_data[directory].checked)
								{
									jetapi.addQueueDownload({ _id: $scope.directory_data[directory].selected_id }, function(response, params) {
										$scope.directory_data[params.directory].checked = false;

										if(response.success && response.data.added) {
											$scope.backups[params.directory][$scope.directory_data[params.directory].selected_id].queue.download = response.data._id;
											$scope.directory_data[params.directory].download = response.data._id;
										} else {
											failed.push($scope.LANG.t('Directory "%s"', params.directory) + ": " + response.message);

											$scope.status = {
												message: failed.join("<br />"),
												type: "danger",
												closeable: false,
												ttl: 10000
											};
										}
									}, {directory: directory});
								}
							}

							$scope.checkall = false;
							$scope.checked = false;
							SweetAlert.success($scope.LANG.t("Success!"), $scope.LANG.t("Your request to add the selected directories to the download queue finished successfully. Please check the top of this page for any errors processing this request."));
						}
						else
						{
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Download cancelled"));
						}

					});
				};

				$scope.$watch("filter", function(){
					if(!$scope.directories_loaded) return;

					$scope.checkall = false;
					$scope.directory_data = {};
					$scope.fetchData();
				}, true);


				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingDirectories = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter(),
						find: {}
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();
					if($scope.filter.destination) apiParams.find.dest = $scope.filter.destination;
					if($scope.filter.structure) apiParams.find.backup_flags = $scope.filter.structure;

					$scope.directories = [];

					jetapi.listDirectories(apiParams, function (response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.directories);
						$scope.directories = response.data.directories;

						for(var i = 0; i < $scope.directories.length; i++) {
							$scope.directories[i].identity = $scope.directories[i].name + $scope.directories[i].type;
							$scope.directories[i].type_name = '';
							if($scope.directories[i].type == CONSTS.BACKUP_TYPE_DIRECTORIES_FILES) $scope.directories[i].type_name = 'file';
							if($scope.directories[i].type == CONSTS.BACKUP_TYPE_DIRECTORIES_DIRS)  $scope.directories[i].type_name = 'directory';
						}

						$scope.directories_loaded = true;
						$scope.listBackups();
						$scope.loadingDirectories = false;
					});
				};

				$scope.calculatingBackupSize = '';
				$scope.calculateBackupSize = function (directory, _id) {

					if($scope.calculatingBackupSize) return;

					$scope.calculatingBackupSize = _id;

					jetapi.calculateBackupSize({ _id: _id }, function(response) {
						$scope.calculatingBackupSize = '';
						if(!response.success) {
							SweetAlert.error($scope.LANG.t("Error!"), response.message);
							return;
						}
						$scope.backups[directory][_id].size = response.data.size;
					});
				};

				$scope.listBackups = function(callback) {

					$scope.resetBackupList();
					$scope.loadingDirectories = true;

					var apiParams = {
						type: CONSTS.BACKUP_TYPE_DIRECTORIES,
						sort: { created: -1 },
						find: {},
						directories: []
					};

					if($scope.filter.destination) apiParams.find.dest = $scope.filter.destination;
					if($scope.filter.structure) apiParams.find.backup_flags = $scope.filter.structure;
					for(var i = 0; i < $scope.directories.length; i++) apiParams.directories.push($scope.directories[i].name);

					jetapi.listBackups(apiParams, function (response) {

						for(var i = 0; i < response.data.backups.length; i++)
						{
							var backup = response.data.backups[i];
							var directory = backup.name + backup.type;

							backup.display = $scope.LANG.d(backup.created) + (backup.encrypted ? ' - ' + $scope.LANG.t('Encrypted') : '');


							if($scope.backups[directory] === undefined) $scope.backups[directory] = {};
							$scope.backups[directory][backup._id] = backup;

							if($scope.directory_data[directory] === undefined) $scope.directory_data[directory] = { checked: false, restore: false, download: false };
							if($scope.directory_data[directory].selected_id === undefined) $scope.directory_data[directory].selected_id = backup._id;
							if(backup.queue.restore) $scope.directory_data[directory].restore = backup.queue.restore;
							if(backup.queue.download) $scope.directory_data[directory].download = backup.queue.download;
						}

						$scope.loadingDirectories = false;
						if(callback !== undefined) callback(response);

					});
				};

				$scope.resetBackupList = function() {

					if(!$scope.directories_loaded) return;

					$scope.checked = false;
					$scope.backups = {};
					//$scope.directory_data= {};

					for(var i = 0; i < $scope.directories.length; i++) {
						if($scope.directory_data[$scope.directories[i].identity] === undefined) $scope.directory_data[$scope.directories[i].identity] = { checked: false, restore: false, download: false };
						$scope.backups[$scope.directories[i].identity] = {};
					}
				};

				$scope.fetchDestinations = function() {

					jetapi.listDestinations({}, function(response) {

						if(!response.success) return;

						for(var i = 0; i < response.data.destinations.length; i++)
						{
							if(response.data.destinations[i].clone_destination) continue;
							$scope.destinations.push({
								_id: response.data.destinations[i]._id,
								name: response.data.destinations[i].name + " (" + response.data.destinations[i].type + ")"
							});
						}

					});
				};

				$scope.getBackupDownloads = function () {

					jetapi.getBackupDownloads({}, function (response) {

					});
				};

				$timeout($scope.fetchDestinations());
				$timeout($scope.getBackupDownloads());
				$timeout($scope.fetchData());
			}]);
});