'use strict';

define([
	'app',
	'libraries/permissions'
], function(app, PERMISSIONS) {
	app.controller("destinationManage",
		["$uibModal", "$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "utils", "SweetAlert",
			function ($uibModal, $rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, utils, SweetAlert) {
				if(!$scope.jobWizard) $rootScope.$emit('menuItem', 'Destinations');

				$scope.details = {
					type: 'Local',
					disklimit: 95,
					path: '/',
					engine: 1,
					schedules: [],
					time_parsed: '12:00 AM',
					time: 0,
					checked: true
				};

				$scope.saveing = false;
				//$scope.saveData = utils.duplicateObject($scope.details);

				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;

				$scope.schedules = {};
				$scope.schedulesCount = 0;

				$scope.engines = [
					{ label: "JetBackup", value: 1 },
					{ label: "cPanel", value: 2 }
				];

				$scope.types = [
					{ label: "Local", value: 'Local', group: $scope.LANG.t('Backup Destinations') },
					{ label: "SSH", value: 'SSH', group: $scope.LANG.t('Backup Destinations') },
					{ label: "SFTP", value: 'SFTP', group: $scope.LANG.t('Backup Destinations') },
					{ label: "FTP", value: 'FTP', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Rsync.net", value: 'RsyncNet', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Amazon S3", value: 'AmazonS3', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Google Drive", value: 'GoogleDrive', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Dropbox", value: 'Dropbox', group: $scope.LANG.t('Backup Destinations') },
					{ label: "Backblaze", value: 'Backblaze', group: $scope.LANG.t('Backup Destinations') },
					{ label: "SSH cPanel Clone", value: 'CloneCpanel', group: $scope.LANG.t('Clone Destinations') }
				];

				if(!PERMISSIONS.canManageDestinationsAsRoot) {
					$scope.details.type = 'SSH';
					$scope.types.splice(0, 1);
				}

				$scope.disklimits = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: "50%", value: 50 },
					{ label: "55%", value: 55 },
					{ label: "60%", value: 60 },
					{ label: "65%", value: 65 },
					{ label: "70%", value: 70 },
					{ label: "75%", value: 75 },
					{ label: "80%", value: 80 },
					{ label: "85%", value: 85 },
					{ label: "90%", value: 90 },
					{ label: "95%", value: 95 },
					{ label: "96%", value: 96 },
					{ label: "97%", value: 97 },
					{ label: "98%", value: 98 },
					{ label: "99%", value: 99 }
				];

				$scope.retries_rates = [
					{ label: $scope.LANG.t("Unlimited"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 4, value: 4 },
					{ label: 5, value: 5 },
				];

				$scope.connections_rates = [
					{ label: $scope.LANG.t("Unlimited"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 4, value: 4 },
					{ label: 5, value: 5 },
				];

				$scope.toggleEngine = function(details) {
					if(details === undefined) return;
					if(details.checked) details.engine = 1;
					return details.checked;
				};

				$scope.loadDestinationType = function(type, defaults) {
					$scope.saveData = utils.duplicateObject($scope.details);

					if($scope.jobWizard && !$scope.wizardData.destinationId){
						$scope.saveData.name = 'Destination_' + parseInt(new Date().getTime() % 10000);
					}

					if(defaults)
					{
						$scope.saveData.type = type;

						switch(type)
						{
							case 'Local':
								$scope.saveData.path = '/';
								$scope.saveData.rsyncbwlimit = 0;
								break;

							case 'CloneCpanel':
							case 'SSH':
							case 'RsyncNet':
								$scope.saveData.path = './';
								$scope.saveData.port = 22;
								$scope.saveData.timeout = 30;
								$scope.saveData.rsyncbwlimit = 0;
								$scope.saveData.rsyncpreferip = 0;
								$scope.authtype = 1;
								break;

							case 'SFTP':
								$scope.saveData.timeout = 30;
								$scope.saveData.retries = 5;
								$scope.saveData.connections = 2;
								$scope.saveData.port = 22;
								$scope.saveData.path = './';
								break;


							case 'FTP':
								$scope.saveData.timeout = 30;
								$scope.saveData.retries = 5;
								$scope.saveData.connections = 2;
								$scope.saveData.port = 21;
								$scope.saveData.ssl = false;
								$scope.saveData.passive = true;
								break;

							case 'AmazonS3':
								$scope.saveData.path = '';
								$scope.saveData.concurrent = 10;
								$scope.saveData.bandwidth = 0;
								$scope.saveData.chunksize = 8;
								break;

							case 'GoogleDrive':
								$scope.saveData.path = '';
								$scope.saveData.authtype = 1;
								break;

							case 'Backblaze':
								$scope.saveData.path = '';
								$scope.saveData.retries = 3;
								// $scope.saveData.retrytimeout = 1;
								break;

							case 'Dropbox':
								$scope.saveData.path = '';
								break;
						}
					}

					$scope.destinationType = 'templates/jetbackup/app/views/destinations/' + type + '.ptt';

					if(type == "SSH" || type == "CloneCpanel"){
						$scope.authtypeChange = function(v){
							if(v == 1) $scope.saveData.password = "";
							else $scope.saveData.privatekey = "";
						};

						$scope.authtype = $scope.saveData.privatekey ? 1 : 0;
						$scope.authtypeChange($scope.authtype);
					}
				};

				$scope.manageSchedule = function (schedule) {

					var modalInstance = $uibModal.open({
						animation: true,
						size: "lg",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/scheduleManagePopup.ptt',
						controller: 'scheduleManagePopup',
						controllerAs: '$scope',
						resolve: {
							schedule: function() { return schedule; },
							type: function() { return 'reindex'; },
							excludeids: function() { return []; },
						}
					});

					modalInstance.result.then(function(scheduleDetails) {

						if(!scheduleDetails._id)
						{
							SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("No schedule selected"));
							return;
						}

						$scope.schedules[scheduleDetails._id] = scheduleDetails;

						if(schedule !== undefined) {
							for(var i in scheduleDetails) {
								if(schedule[i] !== undefined) schedule[i] = scheduleDetails[i];
							}
						}
						else {
							$scope.saveData.schedules.push({
								_id: scheduleDetails._id,
								name: scheduleDetails.name,
								retain: scheduleDetails.retain,
								next_run: 0
							});
						}

					}, function () {
						//$log.info('Modal dismissed at: ' + new Date());
					});

				};

				$scope.removeSchedule = function (schedule) {

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("Are you sure you want to remove this schedule from this job? Please note that all snapshots assigned to this schedule will be deleted in the next job run."),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.deleteSchedule({ _id: schedule._id }, function (response) {

								if(!response.success) {
									SweetAlert.error($scope.LANG.t("Error!"), response.message);
									return;
								}

								for(var i = 0; i < $scope.saveData.schedules.length; i++) {

									if(schedule._id === $scope.saveData.schedules[i]._id)
									{
										$scope.saveData.schedules.splice(i, 1);
										break;
									}
								}

								SweetAlert.success($scope.LANG.t("Success!"), response.message);
							});

						}

					});

				};

				$scope.fetchDestinationData = function(destination_id) {

					jetapi.getDestination({ '_id': destination_id }, function(response) {
						$scope.details = response.data;
						$scope.details.checked = $scope.details.engine === 1;

						//$scope.saveData = utils.duplicateObject($scope.details);
						$scope.loadDestinationType($scope.details.type);
					});

				};

				$scope.$watch("saveData", function(){
					// console.log(utils.isChanged($scope.saveData, $scope.details, ['_id']));
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('$destroy', function() {
					if($scope.jobWizard) return;
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/destinations');
				};

				$scope.parseTime = function (time) {
					var matches = /^([0-9]+):([0-9]+)\s+(AM+|PM+)$/g.exec(time);
					if(!matches) return 0;
					var output = parseInt(matches[1] + matches[2]);
					if(output >= 1200) output -= 1200;
					if(matches[3] === 'PM') output += 1200;
					return output;
				};

				$scope.saveChanges = function(apply, callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details, ['checked'], function(key) {
						return ($scope.details._id === undefined && key === 'type');
					});

					if(apiParams.time_parsed) apiParams.time = $scope.parseTime(apiParams.time_parsed);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageDestination(apiParams, function(response) {

						$scope.saveing = false;
						if(callback !== undefined && typeof callback === 'function') callback(response);

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);

							if(callback === undefined) {
								if(response.data.region !== undefined) $scope.details.region = response.data.region;
								if(apply) $location.path('/destinationManage/' + response.data._id);
								else $location.path('/destinations');
								SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
							}

						}
						else
						{
							$scope.status = {
								message: response.message,
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				//Handle backupJobWizard broadcast
				$scope.$on($scope.wizardName, function () {
					$scope.service.isSaving = true;
					$scope.saveData['disasterrecovery'] = true;
					$scope.saveChanges(false, function (response) {
						if (response.success) {
							$scope.service.wizardAlert = {
								message: response.data.name+" Destination Created",
								closeable: true,
								type: "success",
								ttl: 4000
							};
						}
						$scope.service.isSaving = false;
						$scope.process(response);
					});
				});

				$scope.fetchSchedules = function(callback) {
					jetapi.listSchedules({}, function(response) {
						if(!response.success) return;

						for(var i = 0; i < response.data.schedules.length; i++)
						{
							$scope.schedules[response.data.schedules[i]._id] = response.data.schedules[i];
							$scope.schedulesCount++;
						}

						if(callback !== undefined && typeof callback === 'function') callback();
					});
				};

				$timeout($scope.fetchSchedules(function () {
					if ($scope.jobWizard) $routeParams.id = $scope.wizardData.destinationId;
					if($routeParams.id) $scope.fetchDestinationData($routeParams.id);
					else $scope.loadDestinationType($scope.details.type, true);
				}));


			}]);
});