'use strict';

define([
	'app',
	], function(app) {
	app.controller("destinations",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Destinations');

				$scope.destinations = [];
				$scope.jobs = {};
				$scope.loadingDestinations = false;
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.filter = '';
				$scope.filterOptions = [
					{ label: $scope.LANG.t('All Destinations'), value: '' },
					{ label: $scope.LANG.t('Local Destinations'), value: 'Local' },
					{ label: $scope.LANG.t('SSH Destinations'), value: 'SSH' },
					{ label: $scope.LANG.t('FTP Destinations'), value: 'FTP' },
					{ label: $scope.LANG.t('SFTP Destinations'), value: 'SFTP' },
					{ label: $scope.LANG.t('Google Drive Destinations'), value: 'GoogleDrive' },
					{ label: $scope.LANG.t('Amazon S3 Destinations'), value: 'AmazonS3' },
					{ label: $scope.LANG.t('Dropbox Destinations'), value: 'Dropbox' },
					{ label: $scope.LANG.t('Backblaze Destinations'), value: 'Backblaze' },
					{ label: $scope.LANG.t('Rsync.net Destinations'), value: 'RsyncNet' },
					{ label: $scope.LANG.t('SSH cPanel Clone Destinations'), value: 'CloneCpanel' }
				];

				$scope.types = {
					'Local': 'Local',
					'SSH': 'SSH',
					'RsyncNet': 'Rsync.net',
					'SFTP': 'SFTP',
					'FTP': 'FTP',
					'GoogleDrive': 'Google Drive',
					'AmazonS3': 'Amazon S3',
					'Dropbox': 'Dropbox',
					'Backblaze': 'Backblaze',
					'CloneCpanel': 'SSH cPanel Clone'
				};

				$scope.loaders = {
					state: false,
					validation: false,
					dr: false,
					hide: false,
					delete: false
				};

				meta = meta.new("destinations");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["name","type","owner","engine","jobs_count","disabled","dr"]);
				meta.setTotalItems($scope.destinations.length);

				function apiDestination(params, callback) {

					if($scope.saveing) return;
					$scope.saveing = true;
					$scope.status = undefined;

					if(!params === undefined) params = {};

					jetapi.manageDestination(params, function(response) {

						$scope.saveing = false;

						if(response.success)
						{
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
						}
						else
						{
							SweetAlert.error($scope.LANG.t("Failed!"), response.message);
						}

						if(callback !== undefined) callback(response);

					});
				}

				$scope.onClickDelete = function(destination) {

					if($scope.saveing) return;

					if(destination.jobs_count > 0)
					{
						SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("This destination has an assigned jobs, Please remove those jobs before deleting this destination."));
						return false;
					}

					$scope.saveing = true;
					$scope.loaders.delete = true;

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("This destination will be permanently deleted!"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.deleteDestination({ _id: destination._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success($scope.LANG.t("Success!"), response.message);
								}
								else
								{
									SweetAlert.error($scope.LANG.t("Failed!"), response.message);
								}

							});
						}
						else
						{
							$scope.saveing = false;
							$scope.loaders.delete = false;
							SweetAlert.error($scope.LANG.t("Cancelled!"), $scope.LANG.t("Destination delete cancelled"));
						}

					});
				};

				$scope.toggleStatus = function(destination) {

					$scope.loaders.state = true;

					var disabled = !destination.disabled;

					apiDestination({
						_id: destination._id,
						action: 'modify',
						disabled: disabled
					}, function(response) {
						$scope.loaders.state = false;
						if(response.success) destination.disabled = disabled;
					});
				};

				$scope.toggleDR = function(destination) {

					$scope.loaders.dr = true;

					var dr = !destination.dr;

					apiDestination({
						_id: destination._id,
						action: 'modify',
						disasterrecovery: dr
					}, function(response) {
						$scope.loaders.dr = false;
						if(response.success) destination.dr = dr;
					});
				};

				$scope.toggleHidden = function(destination) {

					$scope.loaders.hide = true;

					var hidden = !destination.hidden;

					apiDestination({
						_id: destination._id,
						action: 'modify',
						hidden: hidden
					}, function(response) {
						$scope.loaders.hide = false;
						if(response.success) destination.hidden = hidden;
					});
				};

				$scope.validateDestination = function(destination) {

					if($scope.saveing) return;
					$scope.saveing = true;
					$scope.loaders.validation = true;

					jetapi.validateDestination({ _id: destination._id }, function(response) {

						$scope.saveing = false;
						$scope.loaders.validation = false;

						if(response.success)
						{
							SweetAlert.success($scope.LANG.t("Success!"), response.message);
						}
						else
						{
							SweetAlert.error($scope.LANG.t("Failed!"), response.message);
						}

					});

				};

				$scope.listJobs = function(callback) {

					jetapi.listBackupJobs({}, function(response) {

						if(!response.success) {
							if(callback !== undefined) callback();
							return;
						}

						for(var i = 0; i < response.data.jobs.length; i++) {
							if($scope.jobs[response.data.jobs[i].destination] === undefined) continue;
							$scope.jobs[response.data.jobs[i].destination].push(response.data.jobs[i]);
						}

						jetapi.listCloneJobs({}, function(response) {

							if(!response.success) {
								if(callback !== undefined) callback();
								return;
							}

							for(var i = 0; i < response.data.jobs.length; i++) {
								if($scope.jobs[response.data.jobs[i].destination] === undefined) continue;
								$scope.jobs[response.data.jobs[i].destination].push(response.data.jobs[i]);
							}

							if(callback !== undefined) callback();
						});
					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingDestinations = true;
					$scope.status = undefined;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: { type: $scope.filter },
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.destinations = [];

					jetapi.listDestinations(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.destinations);
						$scope.destinations = response.data.destinations;
						for(var i = 0; i < $scope.destinations.length; i++) $scope.jobs[$scope.destinations[i]._id] = [];
						$scope.listJobs(function () { $scope.loadingDestinations = false; });
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});