'use strict';

define([
	'app',
	'libraries/permissions'
	], function(app, PERMISSIONS) {
	app.controller("disasterRecovery",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert", "PermPermissionStore",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert, PermPermissionStore) {

				$scope.disasterRecovery = true;
				$scope.saveing = false;
				$scope.status = undefined;

				$scope.saveToDB = function() {
					sessionStorage.disasterRecovery = JSON.stringify({
						details: $scope.details,
						step: $scope.currentStep
					});
				};

				$scope.steps = [
					{
						title: $scope.LANG.t("What to do next?"),
						description: $scope.LANG.t("Please select whether to start to recover your server from a disaster, continue with the default JetBackup configuration, or keep current configuration"),
						template: 'disasterRecovery/selection.ptt'
					},
					{
						title: $scope.LANG.t("JB Configurations destination"),
						description: $scope.LANG.t("Connect to the destination where the JetBackup configuration files are stored"),
						template: 'disasterRecovery/destination.ptt'
					},
					{
						title: $scope.LANG.t("Available JB configuration files"),
						description: $scope.LANG.t("Select the desired JB configuration file"),
						template: 'disasterRecovery/drbackups.ptt'
					},
					{
						title: $scope.LANG.t("Accounts to recover"),
						description: $scope.LANG.t("Select the desired accounts you want to restore"),
						template: 'disasterRecovery/accounts.ptt'
					},
					{
						title: $scope.LANG.t("Recovery accounts queue"),
						description: $scope.LANG.t("Please hold on while the selected account is being restored"),
						template: 'disasterRecovery/queues.ptt'
					}
				];

				if(sessionStorage.disasterRecovery)
				{
					var disasterRecovery = JSON.parse(sessionStorage.disasterRecovery);
					$scope.details = disasterRecovery.details;
					$scope.currentStep = disasterRecovery.step;
				}
				else
				{
					$scope.currentStep = 0;
					$scope.details = {
						destination_id: '',
						drbackups: [],
						drbackup: ''
					};

					$scope.saveToDB();
				}

				$scope.$watch("details", function(){
					$scope.saveToDB();
				}, true);

				$scope.changeStep = function (step) {
					$scope.currentStep = step;
					$scope.saveToDB();
					var path = 'templates/jetbackup/app/views/';
					$scope.currentStepView = path + $scope.steps[step].template;
				};


				$scope.concurrent = 1;
				jetapi.getSettings({section: 'Performance'}, function (response) {
					if(response.success && response.data.concurrentqueueprocesses)
						$scope.concurrent = response.data.concurrentqueueprocesses;
				});

				$scope.updateConcurrentProcess = function(val) {
					if($scope.concurrent + val <= 0) return;
					$scope.concurrent += val;

					var apiParams = {
						section: 'Performance',
						concurrentqueueprocesses: $scope.concurrent
					};

					jetapi.manageSettings(apiParams, function(response) {
						if(!response.success) console.log(response);
					});
				};

				$scope.changeStep($scope.currentStep);

				$scope.exitDRSure = function (apiCall) {

					if (apiCall === undefined || typeof apiCall !== 'function') apiCall = function (callback) {};

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("Are you sure you want to exit disaster recovery mode?"),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function (isConfirm) {

						if (isConfirm) {
							delete sessionStorage.disasterRecovery;

							apiCall(function (response) {
								if (response.success) {
									window.PAGE.system.drMode = false;
									PERMISSIONS._init();

									for (var i in PERMISSIONS) {
										if (!/^can/.test(i) && !/^is/.test(i) && !/^has/.test(i)) continue;
										PermPermissionStore.definePermission(i, function (name) {
											return PERMISSIONS[name];
										});
									}
									// $location.path('/');
									window.location = 'cgi/addons/jetbackup/index.cgi';
								}
							});
						}
					});
				};

				$scope.freshInstallation = function() {
					$scope.exitDRSure(function (callback) {
						if(callback === undefined || typeof callback !== 'function') callback = function() {};
						jetapi.factoryReset({}, callback);
					});
				};

				$scope.exitWithCurrentData = function(){
					$scope.exitDRSure(function (callback) {
						if(callback === undefined || typeof callback !== 'function') callback = function() {};
						jetapi.exitDisasterRecoveryKeepData({}, callback);
					});
				};

				$scope.exitDR = function() {
					$scope.exitDRSure(function (callback) {
						if(callback === undefined || typeof callback !== 'function') callback = function() {};
						jetapi.exitDisasterRecovery({}, callback);
					});
				};

				$scope.nextStep = function() {
					if($scope.saveing) return;
					$scope.saveing = true;
					$scope.status = undefined;
					$rootScope.$broadcast("nextStep");
				};

				$scope.startOver = function() {
					if($scope.saveing) return;
					delete sessionStorage.disasterRecovery;
					$scope.changeStep(1);
				};

				$scope.$on('error', function (event, message) {
					$scope.saveing = false;
					$scope.status = {
						message: message,
						type: "danger",
						closeable: false,
						ttl: 5000
					};
				});

				$scope.$on('commitNextStep', function (event, data) {
					for(var i in data) $scope.details[i] = data[i];
					$scope.saveing = false;
					$scope.changeStep($scope.currentStep+1);
				});
			}
		]
	);
});