'use strict';

define([
	'app',
	'libraries/consts'
	], function(app, CONSTS) {
	app.controller("hookManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert", "cfpLoadingBar",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert, cfpLoadingBar) {
				$rootScope.$emit('menuItem', 'Hooks');

				$scope.details = {};
				$scope.saveData = {
					name: '',
					positiontype: 1,
					position: 1,
					list: [],
					script: ''
				};

				$scope.saveing = false;

				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;

				$scope.backups = {};
				$scope.clones = {};
				$scope.destinations = {};

				$scope.positions = [
					{ label: $scope.LANG.t("Backup"), value: CONSTS.HOOK_POSITION_BACKUP },
					{ label: $scope.LANG.t("Backup Account"), value: CONSTS.HOOK_POSITION_BACKUP_ACCOUNT },
					{ label: $scope.LANG.t("Clone"), value: CONSTS.HOOK_POSITION_CLONE },
					{ label: $scope.LANG.t("Clone Account"), value: CONSTS.HOOK_POSITION_CLONE_ACCOUNT },
					{ label: $scope.LANG.t("Restore"), value: CONSTS.HOOK_POSITION_RESTORE },
					{ label: $scope.LANG.t("Download"), value: CONSTS.HOOK_POSITION_DOWNLOAD },
					{ label: $scope.LANG.t("Reindex"), value: CONSTS.HOOK_POSITION_REINDEX },
					{ label: $scope.LANG.t("Snapshot"), value: CONSTS.HOOK_POSITION_SNAPSHOT },
					{ label: $scope.LANG.t("Daily Cron"), value: CONSTS.HOOK_POSITION_DAILY_CRON }
				];

				$scope.position_types = [
					{ label: $scope.LANG.t("Pre"), value: CONSTS.HOOK_TYPE_POSITION_PRE },
					{ label: $scope.LANG.t("Post"), value: CONSTS.HOOK_TYPE_POSITION_POST }
				];

				$scope.types = {};
				$scope.types[CONSTS.HOOK_POSITION_RESTORE] = [
					//{label:$scope.LANG.t("Accounts cPanel Configurations"), value: CONSTS.BACKUP_TYPE_CONFIG},
					{label:$scope.LANG.t("Accounts Files"), value: CONSTS.BACKUP_TYPE_FILES},
					{label:$scope.LANG.t("Accounts Databases"), value: CONSTS.BACKUP_TYPE_DATABASES},
					{label:$scope.LANG.t("Accounts Emails"), value: CONSTS.BACKUP_TYPE_EMAILS},
					{label:$scope.LANG.t("Accounts Cron Jobs"), value: CONSTS.BACKUP_TYPE_CRON_JOBS},
					{label:$scope.LANG.t("Accounts DNS Zones"), value: CONSTS.BACKUP_TYPE_DNS_ZONES},
					{label:$scope.LANG.t("Accounts SSL Certificates"), value: CONSTS.BACKUP_TYPE_SSL_CERTIFICATES},
					{label:$scope.LANG.t("Accounts Full"), value: CONSTS.BACKUP_TYPE_FULL}
				];

				$scope.types[CONSTS.HOOK_POSITION_DOWNLOAD] = [];
				for(var i = 0; i < $scope.types[CONSTS.HOOK_POSITION_RESTORE].length; i++) $scope.types[CONSTS.HOOK_POSITION_DOWNLOAD][i] = $scope.types[CONSTS.HOOK_POSITION_RESTORE][i];
				$scope.types[CONSTS.HOOK_POSITION_DOWNLOAD].push({label:$scope.LANG.t("Directories"), value: CONSTS.BACKUP_TYPE_DIRECTORIES});

				$scope.itemsSelection = function () {

					var types = {};
					types[CONSTS.HOOK_POSITION_BACKUP] = { template: 'backupJobsSelection', controller: 'backupJobsSelection', resolve: { jobs: function() { return $scope.saveData.list; } } };
					types[CONSTS.HOOK_POSITION_BACKUP_ACCOUNT] = types[CONSTS.HOOK_POSITION_BACKUP];
					types[CONSTS.HOOK_POSITION_CLONE] = { template: 'cloneJobsSelection', controller: 'cloneJobsSelection', resolve: { jobs: function() { return $scope.saveData.list; } } };
					types[CONSTS.HOOK_POSITION_CLONE_ACCOUNT] = types[CONSTS.HOOK_POSITION_CLONE];
					types[CONSTS.HOOK_POSITION_REINDEX] = { template: 'destinationsSelection', controller: 'destinationsSelection', resolve: { destinations: function() { return $scope.saveData.list; } } };

					if(types[$scope.saveData.position] === undefined) return;

					var modalInstance = $uibModal.open({
						animation: true,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/' + types[$scope.saveData.position].template + '.ptt',
						controller: types[$scope.saveData.position].controller,
						controllerAs: '$ctrl',
						resolve: types[$scope.saveData.position].resolve
					});

					modalInstance.result.then(function(selected) {
						$scope.saveData.list = selected;
					}, function () {
						//$log.info('Modal dismissed at: ' + new Date());
					});
				};

				$scope.searchBackupJobs = function(query) {

					if(query)
					{
						var deferred = $q.defer();

						jetapi.listBackupJobs({ filter: query, sort: { name: 1 } }, function(response) {
							var result = [];
							for(var i = 0; i < response.data.jobs.length; i++) {
								if($scope.saveData.list.indexOf(response.data.jobs[i]._id) >= 0) continue;
								result.push(response.data.jobs[i]);
							}
							deferred.resolve(result);
						});

						return deferred.promise;
					}

					return [];
				};

				$scope.searchCloneJobs = function(query) {

					if(query)
					{
						var deferred = $q.defer();

						jetapi.listCloneJobs({ filter: query, sort: { name: 1 } }, function(response) {
							var result = [];
							for(var i = 0; i < response.data.jobs.length; i++) {
								if($scope.saveData.list.indexOf(response.data.jobs[i]._id) >= 0) continue;
								result.push(response.data.jobs[i]);
							}
							deferred.resolve(result);
						});

						return deferred.promise;
					}

					return [];
				};

				$scope.searchDestinations = function(query) {

					if(query)
					{
						var deferred = $q.defer();

						jetapi.listDestinations({ filter: query, sort: { name: 1 } }, function(response) {
							var result = [];
							for(var i = 0; i < response.data.destinations.length; i++) {
								if($scope.saveData.list.indexOf(response.data.destinations[i]._id) >= 0) continue;
								result.push(response.data.destinations[i]);
							}
							deferred.resolve(result);
						});

						return deferred.promise;
					}

					return [];
				};

				$scope.selectItem = function(item, obj) {
					if(item === undefined) return;
					$scope.saveData.list.push(item._id);
					obj.searchBackupText = '';
					obj.searchCloneText = '';
					obj.searchDestinationText = '';
				};

				$scope.fetchHookData = function(hook_id) {

					jetapi.getHook({ '_id': hook_id }, function(response) {
						$scope.details = response.data;
						$scope.saveData = utils.duplicateObject($scope.details);
					});
				};

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.resetList = function() {
					$scope.saveData.list = [];
				};

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.manageTypesList = function(type) {
					var index = $scope.saveData.list.indexOf(type);
					if(index >= 0) $scope.saveData.list.splice(index, 1);
					else $scope.saveData.list.push(type);
				};

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/hooks');
				};

				$scope.saveChanges = function(apply) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageHook(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(apply) $location.path('/hookManage/' + response.data._id);
							else $location.path('/hooks');
							SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
						}
						else
						{
							$scope.status = {
								message: response.message,
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};


				jetapi.listBackupJobs({}, function(response) {
					for(var i = 0; i < response.data.jobs.length; i++) $scope.backups[response.data.jobs[i]._id] = response.data.jobs[i];
				});

				jetapi.listCloneJobs({}, function(response) {
					for(var i = 0; i < response.data.jobs.length; i++) $scope.clones[response.data.jobs[i]._id] = response.data.jobs[i];
				});

				jetapi.listDestinations({}, function(response) {
					for(var i = 0; i < response.data.destinations.length; i++) $scope.destinations[response.data.destinations[i]._id] = response.data.destinations[i];
				});

				var hook_id = $routeParams.id;
				if(hook_id) $timeout($scope.fetchHookData(hook_id));
				else cfpLoadingBar.complete();
	}]);
});