'use strict';

define([
	'app',
	'libraries/permissions',
	'libraries/consts'
	], function(app, PERMISSIONS, CONSTS) {
	app.controller("jobManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert","filterManager",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert, filterManager) {

				if(!$scope.settingsPage) {
					$scope.details = {
						schedules: [],
						filters: []
					};
					$scope.saveData = {
						name: '',
						destination: '',
						options: 0,
						flag: 1,
						time_parsed: '12:00 AM',
						time: 0,
						limitaccounts: 0,
						maximumruntime: 0,
						schedules: [],
						filters: [],
						includelist: [],
						excludelist: [],
						performance: {lve:false,ionice:false,renice:false},
						monitor: {ranfor:0,notran:0},
						loadpauserate: 5,
						loadpausetime: 0,
						retrytimes: 0
					};
				}

				$scope.type = $location.path().indexOf('clone') >= 0 ? 'clone' : 'backup';
				$scope.excluderow = '';
				$scope.includerow = '';
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.jobId = $routeParams.id;

				$scope.filtersConditions = [
					{_id:1,name:$scope.LANG.t("OR")},
					{_id:2,name:$scope.LANG.t("AND")}
				];
				$scope.filtersStructure = [];

				$scope.$watch("filtersStructure", function(){
					$scope.buildFilters(true);
				}, true);

				$scope.removeFilter = function(index) {
					$scope.filtersStructure = filterManager.removeFilter($scope.filtersStructure, index);
				};

				$scope.upFilter = function(index) {
					if(index === 0) return;

					var clickedValue = $scope.filtersStructure[index];
					var upValue = $scope.filtersStructure[index-1];

					if(index === 1) {
						upValue.cond = clickedValue.cond;
						delete clickedValue.cond;
					}

					$scope.filtersStructure[index] = upValue;
					$scope.filtersStructure[index-1] = clickedValue;
				};

				$scope.downFilter = function(index) {
					if(index === ($scope.filtersStructure.length-1)) return;

					var clickedValue = $scope.filtersStructure[index];
					var downValue = $scope.filtersStructure[index+1];

					if(index === 0) {
						clickedValue.cond = downValue.cond;
						delete downValue.cond;
					}

					$scope.filtersStructure[index] = downValue;
					$scope.filtersStructure[index+1] = clickedValue;
				};

				$scope.buildFilters = function(revert) {
					if(revert === undefined) revert = false;

					if(revert) {
						$scope.saveData.filters = filterManager.buildFilters($scope.filtersStructure);
					} else {
						$scope.filtersStructure = filterManager.reBuildStructure($scope.saveData.filters);
					}
				};

				$scope.destinations = [{_id: '', name: $scope.LANG.t('- Select Destination -') }];
				$scope.destinationsIds = {};
				$scope.schedules = {};
				$scope.schedulesCount = 0;
				$scope.scheduleTimeMismatch = false;
				$scope.filters = {};
				$scope.filtersCount = 0;

				$scope.iptypes = [
					{ label: $scope.LANG.t("Restore the account with shared IP"), value: 1 },
					{ label: $scope.LANG.t("Try to match dedicated IP status (Only if possible)"), value: 2 }
				];

				$scope.filterSelection = function () {

					var modalInstance = $uibModal.open({
						animation: true,
						size: "lg",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/accountFilterGroupSelection.ptt',
						controller: 'accountFilterGroupSelection',
						controllerAs: '$scope',
						resolve: {
							filters: function() { return $scope.filters; } ,
							details: function() { return $scope.details; }
						}
					});

					modalInstance.result.then(function(filter_id) {

						if(!filter_id) return;
						var data = { _id: filter_id };
						if($scope.filtersStructure.length > 0) data.cond = 1;
						$scope.filtersStructure.push(data);

					}, function () {
						//$log.info('Modal dismissed at: ' + new Date());
					});
				};

				$scope.manageSchedule = function (schedule) {

					var modalInstance = $uibModal.open({
						animation: true,
						size: "lg",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/scheduleManagePopup.ptt',
						controller: 'scheduleManagePopup',
						controllerAs: '$scope',
						resolve: {
							schedule: function() { return schedule; },
							excludeids: function() { return $scope.jobId ? [$scope.jobId] : []; },
							type: function() { return $scope.saveData.backuptype; }
						}
					});

					modalInstance.result.then(function(scheduleDetails) {

						if(!scheduleDetails._id)
						{
							SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("No schedule selected"));
							return;
						}

						$scope.schedules[scheduleDetails._id] = scheduleDetails;

						if(schedule !== undefined) {
							for(var i in scheduleDetails) {
								if(schedule[i] !== undefined) schedule[i] = scheduleDetails[i];
							}
						}
						else {
							$scope.saveData.schedules.push({
								_id: scheduleDetails._id,
								name: scheduleDetails.name,
								retain: scheduleDetails.retain,
								next_run: 0
							});
						}

						$scope.checkScheduleTimeMismatch();

					}, function () {
						//$log.info('Modal dismissed at: ' + new Date());
					});

				};

				$scope.scheduleSelection = function (schedule) {

					var schedulesTypesInUse = [];

					for(var i = 0; i < $scope.details.schedules.length; i++) {
						if($scope.schedules[$scope.details.schedules[i]._id] === undefined) continue;
						schedulesTypesInUse.push($scope.schedules[$scope.details.schedules[i]._id].type);
					}

					var modalInstance = $uibModal.open({
						animation: true,
						size: "lg",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/scheduleSelection.ptt',
						controller: 'scheduleSelection',
						controllerAs: '$scope',
						resolve: {
							schedules: function() { return $scope.schedules; } ,
							schedule: function() { return schedule; },
							details: function() { return $scope.saveData; },
							type: function() { return $scope.type; }
						}
					});

					modalInstance.result.then(function(scheduleDetails) {

						if(!scheduleDetails._id || ($scope.type == 'backup' && scheduleDetails.retain <= 0))
						{
							if($scope.type == 'backup') SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("No schedule and/or backups retain selected"));
							else SweetAlert.error($scope.LANG.t("Failed!"), $scope.LANG.t("No schedule selected"));
							return;
						}

						if(schedule !== undefined)
						{
							for(var i in scheduleDetails) schedule[i] = scheduleDetails[i];
						}
						else
						{
							$scope.saveData.schedules.push(scheduleDetails);
						}

						$scope.checkScheduleTimeMismatch();

					}, function () {
						//$log.info('Modal dismissed at: ' + new Date());
					});
				};

				$scope.checkScheduleTimeMismatch = function () {
					var time = 0;

					for(var i = 0; i < $scope.saveData.schedules.length; i++)
					{
						if($scope.schedules[$scope.saveData.schedules[i]._id] === undefined) return;
						var currentSchedule = $scope.schedules[$scope.saveData.schedules[i]._id];
						if([1,2,3,4].indexOf(currentSchedule.type) < 0) continue;

						if(currentSchedule.type === 1)
						{
							// TODO validate hourly times
							continue;
						}

						if(!time) time = currentSchedule.time;

						if(currentSchedule.time !== time)
						{
							$scope.scheduleTimeMismatch = true;
							return;
						}
					}

					$scope.scheduleTimeMismatch = false;
				};

				$scope.removeSchedule = function (schedule) {

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("Are you sure you want to remove this schedule from this job? Please note that all snapshots assigned to this schedule will be deleted in the next job run."),
						icon: "warning",
						buttons: [$scope.LANG.t("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.deleteSchedule({ _id: schedule._id }, function (response) {

								if(!response.success) {
									SweetAlert.error($scope.LANG.t("Error!"), response.message);
									return;
								}

								for(var i = 0; i < $scope.saveData.schedules.length; i++) {

									if(schedule._id === $scope.saveData.schedules[i]._id)
									{
										$scope.saveData.schedules.splice(i, 1);
										$scope.checkScheduleTimeMismatch();
										break;
									}
								}

								SweetAlert.success($scope.LANG.t("Success!"), response.message);
							});

						}

					});

				};

				$scope.parseTime = function (time) {
					var matches = /^([0-9]+):([0-9]+)\s+(AM+|PM+)$/g.exec(time);
					if(!matches) return 0;
					var output = parseInt(matches[1] + matches[2]);
					if(output >= 1200) output -= 1200;
					if(matches[3] === 'PM') output += 1200;
					return output;
				};

				$scope.validateListPath = function (path, allowPatterns) {
					if(!path.trim()) return false;
					if(allowPatterns) 
						return !CONSTS.PATH_FILTER_PATTERNS.test(path);
					return !CONSTS.PATH_FILTER.test(path);
				};

				$scope.typeChecked = function(type, types) {
					return ( type & types );
				};

				$scope.toggleOptionsTypes = function(option) {

					if($scope.typeChecked(option, $scope.saveData.options))
					{
						$scope.saveData.options ^= option;
					}
					else
					{
						$scope.saveData.options |= option;
					}
				};

				$scope.checkChange = function() {
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				};

				$scope.$watch("saveData", $scope.checkChange, true);
				$scope.$watch("details", $scope.checkChange, true);

				$scope.validateSchedules = function(schedules, callback) {
					//If in job wizard
					if(schedules.length || $scope.jobWizard) return callback(true);

					SweetAlert.swal({
						title: $scope.LANG.t("Are you sure?"),
						text: $scope.LANG.t("We noticed that you didn't set any schedule for this job. That means that this job will never run automatically, only manually"),
						icon: "warning",
						buttons: [$scope.LANG.t("No, let me set schedules"), true]
					}, callback);
				};

				$scope.fetchSchedules = function(callback) {
					jetapi.listSchedules({}, function(response) {

						if(response.success) {
							for (var i = 0; i < response.data.schedules.length; i++) {
								$scope.schedules[response.data.schedules[i]._id] = response.data.schedules[i];
								$scope.schedulesCount++;
							}
						}

						if(callback !== undefined && typeof callback === 'function') callback();
					});
				};

				$scope.fetchFilters = function(callback) {
					jetapi.listAccountFilterGroups({}, function(response) {

						if(response.success) {
							for(var i = 0; i < response.data.groups.length; i++) {
								$scope.filters[response.data.groups[i]._id] = response.data.groups[i];
								$scope.filtersCount++;
							}
						}

						if(callback !== undefined && typeof callback === 'function') callback();
					});
				};

				$scope.fetchDestinations = function() {
					jetapi.listDestinations({}, function(response) {

						if(!response.success) return;

						for(var i = 0; i < response.data.destinations.length; i++)
						{
							$rootScope.$broadcast('validateDestination', {
								destination: response.data.destinations[i],
								callback: function(destination) {
									if(destination === undefined) return;
									$scope.destinationsIds[destination._id] = destination;
									$scope.destinations.push({
										_id: destination._id,
										name: destination.name + " (" + destination.type + ")"
									});
								}
							});
						}
					});
				};

				$scope.updateData = function(details) {
					$scope.details = details;
					$scope.saveData = utils.duplicateObject($scope.details);
					$scope.checkScheduleTimeMismatch();
					$scope.buildFilters();
				};

				$timeout($scope.fetchFilters(function () {
					$scope.fetchSchedules(function () {
						$timeout($scope.fetchDestinations());

						if(PERMISSIONS.isRoot) {
							jetapi.getSettings({ section: 'resource' }, function (response) {

								if(!response.success) return;

								if (!$routeParams.id && !$scope.settingsPage)
								{
									if(response.data.lveon) {
										$scope.saveData.performance.lve = true;
									} else {
										$scope.saveData.performance.ionice = response.data.ioniceon;
										$scope.saveData.performance.renice = response.data.reniceon;
									}
								}
								$scope.lveinstalled = response.data.lveinstalled;
							});
						}

						var jobID;
						if($scope.jobWizard && $scope.wizardData.jobId)
							jobID = $scope.wizardData.jobId;
						else if ($routeParams.id) jobID = $routeParams.id;

						if (jobID) $timeout($rootScope.$broadcast('fetchJobData', {
							_id: jobID,
							callback: $scope.updateData
						}), 100);
					})
				}));
			}
		]
	);
});