'use strict';

define([
	'app',
	'libraries/consts',
	], function(app, CONSTS) {
	app.controller("restoreConditionManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'Settings');

				$scope.CONSTS = CONSTS;

				$scope.details = {};
				$scope.saveData = {
					type: 1,
					applyto:
						$scope.CONSTS.APPLY_CONDITION_TO_CPANEL |
						$scope.CONSTS.APPLY_CONDITION_TO_WHM,
					title: '',
					content: ''
				};

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;

				$scope.types = [
					{value: 1, label: $scope.LANG.t('All Restore Types') },
					{value: 2, label: $scope.LANG.t('Full Account Restore') },
					{value: 3, label: $scope.LANG.t('File Restore') },
					{value: 4, label: $scope.LANG.t('Database Restore') },
					{value: 5, label: $scope.LANG.t('Email Restore') },
					{value: 6, label: $scope.LANG.t('DNS Zone Restore') },
					{value: 7, label: $scope.LANG.t('Cron Job Restore') },
					{value: 8, label: $scope.LANG.t('SSL Certificate Restore') }
				];

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/restoreConditions');
				};

				$scope.saveChanges = function(apply) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);

					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageRestoreCondition(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(apply) $location.path('/restoreConditionManage/' + response.data._id);
							else $location.path('/restoreConditions');
							SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
						}
						else
						{
							$scope.status = {
								message: response.message,
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};


				$scope.applyChecked = function(apply) {
					return ( apply & $scope.saveData.applyto );
				};

				$scope.toggleCondApply = function(apply) {
					if($scope.applyChecked(apply, $scope.saveData.applyto))
						$scope.saveData.applyto ^= apply;
					else
						$scope.saveData.applyto |= apply;
				};


				$scope.fetch = function(condition_id) {
					jetapi.getRestoreCondition({ '_id': condition_id }, function(response) {
						$scope.details = response.data;
						$scope.saveData = utils.duplicateObject($scope.details);
					});

				};

				var condition_id = $routeParams.id;
				if(condition_id) $timeout($scope.fetch(condition_id));
			}
		]
	);

});