'use strict';

define([
	'app',
	'libraries/permissions'
	], function(app, PERMISSIONS) {
	app.controller("settings",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "jetapi", "meta", "SweetAlert", "utils", "cfpLoadingBar",
			function ($rootScope, $scope, $routeParams, $location, $timeout, jetapi, meta, SweetAlert, utils, cfpLoadingBar) {
				$rootScope.$emit('menuItem', 'Settings');

				if(!PERMISSIONS.isRoot) {
					$location.path("/");
					return;
				}

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.loading = false;

				$scope.details = {};
				$scope.saveData = {};

				$scope.sections = [
					{ _id: 'general', 		name: $scope.LANG.t('General'), 		 template: 'general.ptt', 		icon: 'fa-cogs', 		docs:'/Settings/general.html' },
					{ _id: 'performance', 	name: $scope.LANG.t('Performance'), 	 template: 'performance.ptt',	icon: 'fa-sliders-h',  	docs:'/Settings/performance.html' },
					{ _id: 'resource', 		name: $scope.LANG.t('Resource'), 		 template: 'resource.ptt', 		icon: 'fa-microchip', 	docs:'/Settings/resource.html' },
					{ _id: 'restore', 		name: $scope.LANG.t('Restore'), 		 template: 'restore.ptt', 		icon: 'fa-sync', 		docs:'/Settings/restore.html' },
					{ _id: 'snapshots',		name: $scope.LANG.t('Snapshots'), 		 template: 'snapshots.ptt', 	icon: 'fa-cubes', 		docs:'/Settings/snapshots.html' },
					{ _id: 'notification', 	name: $scope.LANG.t('Notification'), 	 template: 'notification.ptt', 	icon: 'fa-bell', 		docs:'/Settings/notification.html' },
					{ _id: 'binary', 		name: $scope.LANG.t('Binary Locations'), template: 'binary.ptt', 		icon: 'fa-terminal', 	docs:'/Settings/binaryLocations.html'  },
					{ _id: 'gdpr', 			name: $scope.LANG.t('GDPR Compliance'),  template: 'gdpr.ptt', 			icon: 'fa-terminal', 	docs:'/gdpr.html', hidden: true, standAlone: true, class: 'GDPR', description:'GDPR mode allows you to force end users to agree to your user agreement and privacy policy through their cPanel interface.  End users can opt-in for “The right to be forgotten” in which their backups will be automatically terminated in a specific amount of days you select after their account has been terminated. GDPR mode can only be enabled for one destination at a time.  End users will have the ability to select encrypted backups, regular backups or no backups when GDPR mode is enabled.' }
				];

				$scope.getSection = function (section_id) {
					for(var i = 0; i < $scope.sections.length; i++) {
						if($scope.sections[i]._id === section_id) {
							return $scope.sections[i];
						}
					}

					return $scope.sections[0];
				};

				$scope.changeSection = function (section) {

					if($scope.changed)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges(function () {
									$location.path('/settings/' + section._id)
								});
							} else $location.path('/settings/' + section._id);
						});
					}
					else $location.path('/settings/' + section._id);
				};

				$scope.$on('$destroy', function() {
					if($scope.changed)
					{
						SweetAlert.swal({
							title: $scope.LANG.t("Are you sure?"),
							text: $scope.LANG.t("You didn't saved your changes"),
							icon: "warning",
							buttons: [$scope.LANG.t("Disregard Changes"), $scope.LANG.t("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.loadSection = function (section) {
					$scope.currentSection = section;
					$scope.settingsSection = 'templates/jetbackup/app/views/settings/' + section.template;

					if(section.standAlone) {
						$rootScope.$emit('menuItem', section.class);
					} else {
						$rootScope.$emit('menuItem', 'Settings');
					}
				};

				$scope.loadSection($scope.getSection($routeParams.section ? $routeParams.section : 'general'));

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details);
				}, true);

				$scope.saveChanges = function(callback) {
					cfpLoadingBar.start();

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details, undefined, function (key) {
						return key == 'mustagree' || key == 'suspendbackups';
					});
					apiParams.section = $scope.currentSection._id;

					jetapi.manageSettings(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(callback !== undefined) callback();
							SweetAlert.swal($scope.LANG.t("Success!"), response.message, "success");
						}
						else
						{
							$scope.status = {
								message: response.message,
								type: "danger",
								closeable: false,
								ttl: 10000
							};
						}
						cfpLoadingBar.complete();
					});
				};

				$scope.fetch = function() {
					$scope.loading = true;
					jetapi.getSettings({ section: $scope.currentSection._id }, function(response) {
						$scope.loading = false;
						$scope.details = response.data;
						$scope.saveData = utils.duplicateObject($scope.details);
					});
				};

				$timeout($scope.fetch());

			}]);
});