'use strict';

define([
	'app',
	'libraries/consts'
	], function(app, CONSTS) {
	app.controller("gdprSettings",
		["$rootScope", "$scope", "$routeParams", "$uibModal", "$location", "$timeout", "jetapi", "utils",
			function ($rootScope, $scope, $routeParams, $uibModal, $location, $timeout, jetapi, utils) {

				$scope.settingsPage = true;
				$scope.saveData.includelist = [];
				$scope.saveData.excludelist = [];
				$scope.saveData.schedules = [];
				$scope.exclude = {data:''};

				$scope.flags = [
					{ label: $scope.LANG.t("Archived"), value: 2 },
					{ label: $scope.LANG.t("Compressed"), value: 4 }
				];

				$scope.validateListPath = function (path, allowPatterns) {
					if(!path.trim()) return false;
					if(allowPatterns)
						return !CONSTS.PATH_FILTER_PATTERNS.test(path);
					return !CONSTS.PATH_FILTER.test(path);
				};

				$scope.addExcludeRow = function () {
					if(!$scope.exclude.data.trim()) return;

					if($scope.validateListPath($scope.exclude.data.trim(), true))
						$scope.saveData.excludelist.push($scope.exclude.data.trim());

					$scope.exclude.data = '';
				};

				$scope.addMultiListRow = function(type, list) {

					if(type == 'include') {
						$scope.listTitle = $scope.LANG.t("Directories and Files to include");
						$scope.listData = $scope.saveData.includelist.join("\n");
					} else {
						$scope.listTitle = $scope.LANG.t("Directories and Files to excludes");
						$scope.listData = $scope.saveData.excludelist.join("\n");
					}

					$scope.listUIB = $uibModal.open({
						animation: true,
						scope: $scope,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/listSelection.ptt'
					});

					$scope.listUIB.result.then(function(records) {
						while(list.length) list.pop();
						records = records.split("\n");
						for(var i = 0; i < records.length; i++) {
							if($scope.validateListPath(records[i].trim(), true)) list.push(records[i].trim());
						}
					});
				};

				$scope.parseTime = function (time) {
					var matches = /^([0-9]+):([0-9]+)\s+(AM+|PM+)$/g.exec(time);
					if(!matches) return 0;
					var output = parseInt(matches[1] + matches[2]);
					if(output >= 1200) output -= 1200;
					if(matches[3] === 'PM') output += 1200;
					return output;
				};

				$scope.$on('validateDestination', function(event, data) {
					if(data.destination.clone_destination) data.callback();
					else data.callback(data.destination);
				});

				$scope.saveChanges = function(callback) {

					var apiParams = utils.saveParams($scope.saveData, $scope.details);
					if(apiParams.time_parsed) $scope.saveData.time = $scope.parseTime(apiParams.time_parsed);

					if(apiParams.useragreement !== undefined || apiParams.privacypolicy !== undefined || apiParams.destinationinfo !== undefined) {

						$uibModal.open({
							animation: true,
							size: "lg",
							ariaLabelledBy: 'modal-title',
							ariaDescribedBy: 'modal-body',
							templateUrl: 'templates/jetbackup/app/views/gdprMustAgree.ptt',
							controller: 'gdprMustAgree',
							controllerAs: '$scope',
							resolve: {}
						}).result.then(function (value) {

							$scope.saveData.mustagree = !!value.mustagree;
							$scope.saveData.suspendbackups = !!value.suspendbackups;

							$scope.$parent.saveChanges(callback);
						});

					} else {
						$scope.$parent.saveChanges(callback);
					}
				};

				$scope.delete_data_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 5, value: 5 },
					{ label: 7, value: 7 },
				];

				$scope.runtime_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 60, value: 60 },
					{ label: 120, value: 120 },
					{ label: 300, value: 300 },
					{ label: 600, value: 600 },
				];

				$scope.monitor_runfor_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 5, value: 5 },
					{ label: 7, value: 7 },
				];

				$scope.monitor_notran_ttl_rates = [
					{ label: $scope.LANG.t("Disabled"), value: 0 },
					{ label: 1, value: 1 },
					{ label: 2, value: 2 },
					{ label: 3, value: 3 },
					{ label: 5, value: 5 },
					{ label: 7, value: 7 },
				];
			}
		]
	);
});