'use strict';

define([
	'app',
	'libraries/utils',
	'libraries/consts',
], function (app, UTILS, CONSTS) {
	app.controller("selectAccount",
		["$rootScope", "$scope", "$timeout", "SweetAlert", "jetapi", "$q", "$uibModal",
			function ($rootScope, $scope, $timeout, SweetAlert, jetapi, $q, $uibModal) {


				$scope.UTILS = UTILS;
				$scope.data = {accountList: []};

				if ($scope.service) {
					$scope.service.showClose = true;
					$scope.service.hideNext = $scope.service.setFinal = false;
					if($scope.wizardData.accounts) $scope.data.accountList = $scope.wizardData.accounts;
				}

				$scope.selectItem = function (acct, obj) {
					if (acct) $scope.data.accountList.push(acct)
					obj.searchItemText = acct;
				};

				var getCallParams = function(filter) {
					var params = {
						sort: 			{_id: 1},
						find: {}
					};
					params.type = $scope.wizardData.type ? $scope.wizardData.type: CONSTS.BACKUP_TYPE_FULL;
					if(filter) params.filter = filter;
					if($scope.wizardData.destination) params.find.dest = $scope.wizardData.destination;
					if($scope.wizardData.structure) params.find.backup_flags = $scope.wizardData.structure;

					return params;
				};

				$scope.searchAccounts = function (query) {
					if (query) {
						var accounts = [];
						var deferred = $q.defer();
						jetapi.listAccountsByBackupFilters(getCallParams(query), function (response) {
							for (var i = 0; i < response.data.accounts.length; i++) {
								var account = response.data.accounts[i].username;
								if($scope.data.accountList.indexOf(account) >= 0) continue;
								accounts.push(response.data.accounts[i]);
							}

							deferred.resolve(accounts);
						});
						return deferred.promise;
					}
					return [];
				};

				$scope.accountsSelect = function () {
					var modalInstance = $uibModal.open({
						animation: true,
						size: "lg",
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: 'templates/jetbackup/app/views/accountsSelection.ptt',
						controller: 'accountsSelection',
						controllerAs: '$scope',
						resolve: {
							accounts: function() {
								return $scope.data.accountList;
							},
							callParams: function () {
								return getCallParams();
							}
						}
					});

					modalInstance.result.then(function (selected) {
						$scope.data.accountList = selected;
					},
					function () {
					});
				};

				$scope.$watch('data.accountList', function () {
					if($scope.data.accountList.length > 1 ) $scope.service.wizardAlert = null;
				}, true);

				$scope.$on($scope.wizardName, function (e,v) {
					if($scope.data.accountList.length < 1) {
						$scope.service.wizardAlert = {
							message: 'Please select account',
							closeable: true,
							type: "danger",
							ttl: 2000
						};
						return;
					}
					$scope.process({success: true, data:{accounts:$scope.data.accountList}});

				});
			}
		]);
});