'use strict';

define([
	'app',
	'libraries/consts',
], function (app, CONSTS) {
	app.controller("selectBackup",
		["$rootScope", "$scope", "$templateCache", "SweetAlert", "jetapi",
			function ($rootScope, $scope, $templateCache, SweetAlert, jetapi) {


				$scope.CONSTS = CONSTS;

				if ($scope.service) {
					$scope.service.showClose = true;
					$scope.currentAccount = $scope.wizardData.accounts.pop();
					if($scope.wizardData.accounts.length <=0) $scope.service.setFinal = $scope.service.hideNext = true;
					$scope.wizardData.accounts.push($scope.currentAccount);

					$scope.data = {};
					$scope.current = '';
					$scope.fileManagerTemplate = '';
				}

				$scope.backupTypes = {};
				$scope.backupTypes[CONSTS.BACKUP_TYPE_FULL] 		= $scope.LANG.t("Full");
				$scope.backupTypes[CONSTS.BACKUP_TYPE_FILES] 		= $scope.LANG.t("Files");
				$scope.backupTypes[CONSTS.BACKUP_TYPE_DATABASES] 	= $scope.LANG.t("Database");
				$scope.backupTypes[CONSTS.BACKUP_TYPE_EMAILS] 		= $scope.LANG.t("Email");
				$scope.backupTypes[CONSTS.BACKUP_TYPE_CRON_JOBS] 	= $scope.LANG.t("Cron Job");
				$scope.backupTypes[CONSTS.BACKUP_TYPE_DNS_ZONES] 	= $scope.LANG.t("DNS Zone");

				var params = {
					accounts: [$scope.currentAccount],
					type: $scope.wizardData.type,
					find: {backup_flags: $scope.wizardData.structure},
					sort: {created: -1},
				};

				jetapi.listBackups(params, function (response) {

					if (!response.success || response.data.backups.length <= 0) { //TODO Set Error
						$scope.service.hideNext = true;
						return;
					}

					switch ($scope.wizardData.type) {

						case CONSTS.BACKUP_TYPE_FULL:
						case CONSTS.BACKUP_TYPE_FILES:
						case CONSTS.BACKUP_TYPE_CRON_JOBS:

							$scope.data.selectedSnap = '';
							$scope.backups = [{_id: '', created: '-- Please Select Backup --'}];
							for (var i=0; i < response.data.backups.length; i++) {
								response.data.backups[i].created =  $scope.LANG.d(response.data.backups[i].created);
								$scope.backups.push(response.data.backups[i]);
							}
							$scope.selectedSnap = '';
							break;

						case CONSTS.BACKUP_TYPE_DATABASES:
						case CONSTS.BACKUP_TYPE_EMAILS:
						case CONSTS.BACKUP_TYPE_DNS_ZONES:

							var listBackupsByType = {};
							$scope.accountItems = [];
							$scope.data.selectedSnap = {};

							var backups = response.data.backups;

							for (var i=0; i < backups.length; i++) {
								var backup = backups[i];

								backup.created = $scope.LANG.d(backup.created);

								if (listBackupsByType[backup.name] === undefined)
									listBackupsByType[backup.name] = [{_id: '', created: '-- Select Backup --'}];

								listBackupsByType[backup.name].push(backup);
							}

							for (var type_name in listBackupsByType) {

								$scope.accountItems.push({
									name: type_name,
									items: listBackupsByType[type_name],
									selected: ''
								});
							}

							break;
					}


				});


				$scope.loadFileManager = function () {
					$scope.fileManagerTemplate = 'templates/jetbackup/app/views/wizard/fileManager.ptt?v=' + Date.now();
				};

				var restoreQueueConfirmation = function(callback) {

					SweetAlert.swal({
						title: $scope.LANG.t("Continue?"),
						text: $scope.LANG.t("The selected item will be send to the restore queue"),
						icon: "warning",
						buttons: [$scope.LANG.t("NO"), $scope.LANG.t("YES")]
					}, function (isConfirm) {
						if(isConfirm) {
							$scope.wizardData.accounts.pop();
							callback();
						}
					});
				};


				$scope.$on($scope.wizardName, function (env, data) {

					switch ($scope.wizardData.type) {

						case CONSTS.BACKUP_TYPE_FULL:
						case CONSTS.BACKUP_TYPE_CRON_JOBS:
							if (!$scope.data.selectedSnap){
								SweetAlert.error($scope.LANG.t("ERROR!"), $scope.LANG.t("Please Select backup to restore!"));
								return
							}

							restoreQueueConfirmation(function() {
								jetapi.addQueueRestore({_id: $scope.data.selectedSnap}, function (response) {
									if(!response.success){SweetAlert.error($scope.LANG.t("ERROR!"), $scope.LANG.t("Something went wrong!")); return;}
								});
								$scope.process({success: true, data: {accounts: $scope.wizardData.accounts}});
								if ($scope.wizardData.accounts.length > 0) $scope.service.updateRandom();
							});

							break;

						case CONSTS.BACKUP_TYPE_DATABASES:
						case CONSTS.BACKUP_TYPE_EMAILS:
						case CONSTS.BACKUP_TYPE_DNS_ZONES:


							restoreQueueConfirmation( function() {
								// console.log($scope.accountItems);
								// return;
								for(var i=0; i<$scope.accountItems.length; i++) {
									if($scope.accountItems[i].selected) {
										jetapi.addQueueRestore({_id: $scope.accountItems[i].selected}, function (response) {
											if(!response.success) {SweetAlert.error($scope.LANG.t("ERROR!"), $scope.LANG.t("Something went wrong!")); return;}
										})
									}
								}
								$scope.process({success: true, data: {accounts: $scope.wizardData.accounts}});
								if ($scope.wizardData.accounts.length > 0) $scope.service.updateRandom();
							});

							break;

						case CONSTS.BACKUP_TYPE_FILES:
							$scope.$broadcast('buildRestoreQuery', function () {

							});
							break;
					}


				});

				//HANDLE THE FILE RESTORE
				$scope.$on('onClickRestoreConfirm', function (e, apiParams) {
					if(apiParams.files.length <= 0)  {SweetAlert.error($scope.LANG.t("ERROR!"), $scope.LANG.t("Please Select files to restore!")); return;}
					restoreQueueConfirmation(function(){
						jetapi.addQueueRestore(apiParams, function (response ) {
							if(!response.success) {SweetAlert.error($scope.LANG.t("ERROR!"), $scope.LANG.t("Something went wrong!"));}
						});

						$scope.process({success: true, data: {accounts: $scope.wizardData.accounts}});
						if ($scope.wizardData.accounts.length > 0) $scope.service.updateRandom();
					});

				});

			}
		]);
});