'use strict';

define([
	'app',
], function (app) {
	app.directive("search", ["lang", function(lang) {

		return {
			restrict: "E",
			template: '<div class="input-group">\n' +
			'    <input type="text"\n' +
			'        id="{{parentID}}_input"\n' +
			'        class="form-control"\n' +
			'        placeholder="{{placeholder}}"\n' +
			'        title="{{title}}"\n' +
			'        ng-model="filterText"\n' +
			'        ng-keyup="clear($event)"\n' +
			'        ng-debounce\n' +
			'        prevent-default-on-enter\n' +
			'        auto-focus="{{autofocus}}"\n' +
			'        aria-label="' + lang.t("Search") + '"/>\n' +
			'    <span class="input-group-btn">\n' +
			'        <button id="{{parentID}}_submit_btn"\n' +
			'            class="btn btn-default"\n' +
			'            ng-click="filterText=\'\'"\n' +
			'            type="button"\n' +
			'            ng-attr-aria-label="{{ !filterText ? ariaLabelSearch : ariaLabelClear }}">\n' +
			'            <span class="fas"\n' +
			'                ng-class="{ \'fa-search\' : !filterText, \'fa-times\' :filterText  }"\n' +
			'                aria-hidden="true">\n' +
			'            </span>\n' +
			'        </button>\n' +
			'    </span>\n' +
			'</div>',
			require: "ngModel",
			replace: true,
			scope: {
				parentID: "@id",
				placeholder: "@?placeholder",
				autofocus: "@?autofocus",
				title: "@?title"
			},
			compile: function() {

				return {
					pre: function(scope, element, attrs) { // eslint-disable-line no-unused-vars
						if (angular.isUndefined(attrs.placeholder)) {
							attrs.placeholder = lang.t("Search");
						}
						if (angular.isUndefined(attrs.title)) {
							attrs.title = lang.t("Search");
						}

						if (angular.isUndefined(attrs.autofocus)) {
							attrs.autofocus = false;
						} else {
							attrs.autofocus = true;
						}

						scope.autofocus = attrs.autofocus;
						scope.placeholder = attrs.placeholder;
						scope.title = attrs.title;
						scope.ariaLabelSearch = lang.t("Search");
						scope.ariaLabelClear = lang.t("Clear");
					},
					post: function(scope, element, attrs, ctrls) { // eslint-disable-line no-unused-vars
						var ngModelCtrl = ctrls;

						if (!ngModelCtrl) {
							return; // do nothing if no ng-model on the directive
						}

						ngModelCtrl.$render = function() {
							scope.filterText = ngModelCtrl.$viewValue;
						};

						scope.clear = function(event) {
							if (event.keyCode === 27) {
								scope.filterText = "";
							}
						};

						var timeout = null;
						scope.$watch("filterText", function() {
							if(timeout !== null) clearTimeout(timeout);
							timeout = setTimeout(function(){
								ngModelCtrl.$setViewValue(scope.filterText);
							}, 100);
						});

					}
				};
			}
		};
	}]);
});