'use strict';

define(['app'], function (app) {
	app.factory('utils', function () {

		var deepDiffMapper = function() {
			return {
				getChanges: function(original, current, removeParams, skipCallback) {
					if(original === undefined || current === undefined) return {};
					if (!this.isObject(original) || !this.isObject(current)) throw 'Invalid argument. Function given, object expected.';
					var changes = {};
					for(var i in current) {
						if(removeParams !== undefined && removeParams.indexOf(i) >= 0) continue;
						if(i === '_id' || (skipCallback !== undefined && typeof skipCallback === 'function' && skipCallback(i))) changes[i] = current[i];
						else if(this.changedValue(original[i], current[i])) changes[i] = current[i];
					}
					return changes;
				},
				isChanged: function(original, current, exclude) {
					if(original === undefined || current === undefined) return false;
					if (!this.isObject(original) || !this.isObject(current)) throw 'Invalid argument. Function given, object expected.';
					for(var i in current) {
						if(exclude !== undefined && exclude.indexOf(i) >= 0) continue;
						if(this.changedValue(original[i], current[i])){
							return true;
						}
					}
					return false;
				},
				changedValue: function(original, current) {

					if (this.isFunction(original) || this.isFunction(current)) throw 'Invalid argument. Function given, object expected.';

					if (this.isValue(current)) {
						if(/^object:[0-9]+$/.test(current)) return false;
						return original !== current;
					}

					var keys = {};
					for (var ckey in current) {
						keys[ckey] = 1;
						if (this.isFunction(current[ckey])) continue;
						if(original === undefined || original[ckey] === undefined) return true;
						if(this.changedValue(original[ckey], current[ckey])) return true;
					}

					for (var okey in original) if(!keys[okey]) return true;

					return false;
				},
				isFunction: function(obj) {
					return {}.toString.apply(obj) === '[object Function]';
				},
				isArray: function(obj) {
					return {}.toString.apply(obj) === '[object Array]';
				},
				isObject: function(obj) {
					return {}.toString.apply(obj) === '[object Object]';
				},
				isDate: function(obj) {
					return {}.toString.apply(obj) === '[object Date]';
				},
				isValue: function(obj) {
					return !this.isObject(obj) && !this.isArray(obj);
				}
			}
		}();

		return {
			saveParams: function (saveData, details, removeParams, skipCallback) {
				return deepDiffMapper.getChanges(details, saveData, removeParams, skipCallback);
			},
			duplicateObject: function (obj, type) {
				return JSON.parse(JSON.stringify(obj)); 
			/*	
				
				if(type === undefined) type = 'object';
				var newObj = type === 'object' ? {} : [];

				for(var i in obj) {
					switch(typeof obj[i]) {
						default:
							newObj[i] = obj[i];
						break;
						case 'function': break;
						case 'object':
							if(Array.isArray(obj[i]))
							{
								newObj[i] = [];
								for(var j = 0; j < obj[i].length; j++) {
									if(typeof obj[i][j] === 'object') {
										newObj[i][j] = this.duplicateObject(obj[i][j], Array.isArray(obj[i][j]) ? 'array' : 'object');
									}
									else newObj[i][j] = obj[i][j];
								}
							}
							else newObj[i] = Object.assign({}, obj[i]);
						break;
					}
				}

				return newObj;
			*/
			},
			isChanged: function(saveData, details, exclude) {
				return deepDiffMapper.isChanged(details, saveData, exclude);
			}
		};
	});
});