
'use strict';

define([
	'app',
	'libraries/consts',
	'libraries/permissions'
], function (app, CONSTS, PERMISSIONS) {
	app.factory('wizards', ["JWizardService", "SweetAlert", "$location", "jetapi", "lang", function (JWizardService, SweetAlert, $location, jetapi, lang) {


		return {

			backupJobs: function (callback) {

				if(callback === undefined) callback = function () {};

				var STEPS = {
					WIZARD_WELCOME: 2123,
					WIZARD_DESTINATION_PICKER: 15,
					WIZARD_DESTINATION_CREATE: 1,
					WIZARD_ENABLE_DR: 2,
					WIZARD_CREATE_BACKUP: 3,
					WIZARD_MANAGE_FILTERS: 4,
					WIZARD_MANAGE_SCHEDULES: 5,
				};

				var wizard = new JWizardService("wizard1");

				if(PERMISSIONS.canManageDestinations) {
					wizard.setFirstStep({
						name: STEPS.WIZARD_WELCOME,
						tpl: 'wizard/wizardWelcomePage',
						title: lang.t("Backup Job Wizard"),
						// description: '<strong>' + lang.t("Please select a backup job to create from the options below") + '</strong>',
						process: function (data) {
							if (data && !data.next) {
								// wizard.updateStorage('jobId', data.jobId._id);
								wizard.finish();
								return;
							}

							return STEPS.WIZARD_DESTINATION_PICKER;
						},
						hideFooter: true
					});

					wizard.addStep({
						name: STEPS.WIZARD_DESTINATION_PICKER,
						tpl: 'wizard/wizardDestinationPicker',
						title: lang.t("Choose Destination"),
						description: lang.t("A Backup destination is the place holder of your backup files (this can be remote or local)") +
							"<br>" + lang.t("For more information please follow:") +
							"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/Destinations/createNewDestination.html\" target=\"_blank\">" +
							lang.t("Backup Destinations Documentation") + "</a>",
						process: function (data) {
							wizard.updateStorage('destinationId', data.destination_id);
							return data.destination_id ? STEPS.WIZARD_CREATE_BACKUP : STEPS.WIZARD_DESTINATION_CREATE;
						}
					});
				}
				else {
					wizard.setFirstStep({
						name: STEPS.WIZARD_DESTINATION_PICKER,
						tpl: 'wizard/wizardDestinationPicker',
						title: lang.t("Choose Destination (click next to create new)"),
						description: lang.t("A Backup destination is the place holder of your backup files (this can be remote or local)") +
							"<br>" + lang.t("For more information please follow:") +
							"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/Destinations/createNewDestination.html\" target=\"_blank\">" +
							lang.t("Backup Destinations Documentation") + "</a>",
						process: function (data) {
							wizard.updateStorage('destinationId', data.destination_id);
							return data.destination_id ? STEPS.WIZARD_CREATE_BACKUP : STEPS.WIZARD_DESTINATION_CREATE;
						}
					});
				}





				wizard.addStep({
					name: STEPS.WIZARD_DESTINATION_CREATE,
					tpl: 'destinationManage',
					title: lang.t("Manage Destination"),
					description: lang.t("A Backup destination is the place holder of your backup files (this can be remote or local)") +
						"<br>" + lang.t("For more information please follow:") +
						"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/Destinations/createNewDestination.html\" target=\"_blank\">" +
						lang.t("Backup Destinations Documentation") + "</a>",
					process: function (data) {
						wizard.updateStorage('destinationId', data.data._id);
						return STEPS.WIZARD_CREATE_BACKUP;
						// return STEPS.WIZARD_ENABLE_DR;
					}
				});

				wizard.addStep({
					name: STEPS.WIZARD_CREATE_BACKUP,
					tpl: 'backupJobManage',
					title: lang.t("Backup Job Settings"),
					description: "<strong>" + lang.t("JetBackup can perform several types of backups:") + "</strong> <br>" +
						lang.t("Accounts - Copy files/folders from cPanel account that you choose to include (same as \"/scripts/pkgacct\").") + "<br>" +
						lang.t("Directories - Copy server files/folders that you choose to include (not necessarily related to cPanel accounts).") + "<br>" +
						lang.t("Replicate - Copy files/folders \"as is\".") +
						"<br>" + lang.t("For more information please follow:") +
						"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/BackupJobs/backupJobTypes.html\" target=\"_blank\">" +
						lang.t("Backup Job") + "</a>",
					process: function (data) {
						wizard.updateStorage('jobId', data.data._id);
						// return data.data.backuptypes <= 127 ? STEPS.WIZARD_MANAGE_FILTERS : STEPS.WIZARD_MANAGE_SCHEDULES;
						return STEPS.WIZARD_MANAGE_SCHEDULES;
					}
				});

				// wizard.addStep({
				// 	name: STEPS.WIZARD_MANAGE_FILTERS,
				// 	tpl: 'wizard/wizardFilters',
				// 	title: lang.t("Select Account Filters or press next to backup all accounts"),
				// 	description: lang.t("A account filter helps us to backup accounts by predefined conditions.") +
				// 		"<br>" + lang.t("For more information please follow:") +
				// 		"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/ManageAccountFilters/createNewFilter.html\" target=\"_blank\">" +
				// 		lang.t("Filters") + "</a>",
				// 	process: function (data) {
				// 		wizard.updateStorage('filters', data.data.filters);
				// 		return STEPS.WIZARD_MANAGE_SCHEDULES;
				// 	}
				// });

				wizard.addStep({
					name: STEPS.WIZARD_MANAGE_SCHEDULES,
					tpl: 'wizard/wizardSchedule',
					title: lang.t("Create a Schedule"),
					description: lang.t("A backup job schedule is the date and time that your backup job will be executed.") +
						"<br>" + lang.t("For more information please follow:") +
						"<a class=\"wizard-link\" href=\"https://docs.jetbackup.com/manual/whm/CreateSchedules/createSchedule.html\" target=\"_blank\">Schedule Documentation</a>",
					process: function (data) {
						wizard.updateStorage('schedules', data.data.schedules);
						wizard.updateStorage('time_parsed', data.data.time_parsed);
					},
					final: true
				});

				var finish = function () {

					if (!wizard.data.jobId) {
						SweetAlert.success(lang.t("Success!"), lang.t("Backup Job Created Successfully"));
						wizard.cleanStorageData();
						wizard.close();
						callback(true);
						return;
					}

					// wizard.next();
					jetapi.manageBackupJob({
						_id: wizard.data.jobId,
						action: 'modify',
						disabled: 0
					}, function (response) {
						wizard.cleanStorageData();
						wizard.close();
						callback(!!response.success);
						if (!response.success) {
							SweetAlert.error(lang.t("Error!"), response.message);
							return;
						}
						SweetAlert.success(lang.t("Success!"), lang.t("Backup Job Created Successfully"));
					});
				};

				wizard.setFinish(finish);
				wizard.open();

			},
			restore: function (callback) {

				if(callback === undefined) callback = function () {};

				var STEPS = {
					SELECT_TYPE: 	1,
					SELECT_ACCOUNT: 2,
					BACKUP_SELECT: 	3,
					FILES_RESTORE:  4
				};

				var wizard = new JWizardService("restoreWizard");


				wizard.setFirstStep({
					name: STEPS.SELECT_TYPE,
					tpl: 'wizard/selectRestoreType',
					title: lang.t("Restore Wizard"),
					description: lang.t("Please select the backup type, destination and structure."),
					process: function (response) {
						if (!response.success) return;

						wizard.updateStorage('destination', response.data.destination);
						wizard.updateStorage('structure', response.data.structure);
						wizard.updateStorage('type', response.data.type);

						return STEPS.SELECT_ACCOUNT;
					}
					// hideFooter: true
				});

				wizard.addStep({
					name: STEPS.SELECT_ACCOUNT,
					tpl: 'wizard/selectAccount',
					title: lang.t("Select Accounts"),
					process: function (response) {
						if (!response.success) return;
						if (response.data) wizard.updateStorage('accounts', response.data.accounts);
						return STEPS.BACKUP_SELECT;
					}
				});

				wizard.addStep({
					name: STEPS.BACKUP_SELECT,
					tpl: 'wizard/selectBackup',
					title: lang.t("Select Backup"),
					process: function (response) {
						if (!response.success) return;

						if (response.data.accounts.length > 0 ) {
							wizard.updateStorage('accounts', response.data.accounts);
							return STEPS.BACKUP_SELECT;
						}
						else wizard.finish();
					}
				});

				wizard.setFinish(function (fail) {
					if(!fail){
						SweetAlert.swal({
							title: lang.t("Success!"),
							text: lang.t("The selected items successfully added to the restore queue."),
							icon: "success",
							buttons: [lang.t("OK"), lang.t("GO TO QUEUE")]
						}, function (isConfirm) {
							if(isConfirm) $location.path("/queues");
						});
					}
					wizard.cleanStorageData();
					wizard.close();
				});

				wizard.open();

			}
		};

	}]);
});