/*
* base/frontend/paper_lantern/jetbackup/controllers/settings.js
*
* JetBackup @ package
* Created By Idan Ben-Ezra
*
* Copyrights @ JetApps
* https://www.jetapps.com
*
**/

/* global define: false, PAGE: false */

define([
		"app"
    ], function(app) {
        app.controller("gdpr",
            ["$rootScope", "$scope", "$interval", "$timeout", "jetapi", "lang",
                function($rootScope, $scope, $interval, $timeout, jetapi, lang) {

					$scope.status = undefined;
					$scope.settings = {enabled:false};
					$scope.showTerms = false;
					$scope.encryptionKey = '';
					$scope.submitEncryptionKey = '';
					$scope.loading = false;

					$scope.userAgreement = 0;
					$scope.privacyPolicy = 0;
					$scope.backupDestinations = 0;

					$scope.confirmTerms = function() {

						if(!$scope.showTerms) return;

						$scope.status = undefined;

						if(!$scope.userAgreement || !$scope.privacyPolicy || !$scope.backupDestinations){
							$scope.status = {
								message: lang.t("You must agree to all terms in order to continue"),
								type: 'danger',
								closeable: true,
								ttl: 10000
							};
							return;
						}

						jetapi.manageGDPR({ iapprove: 1 }, function (response) {

							if(!response.status) {
								$scope.status = {
									message: lang.t(response.errors[0]),
									type: 'danger',
									closeable: true,
									ttl: 10000
								};

								return;
							}
							$scope.showTerms = false;
							window.PAGE.info.gdpr.termsagreed = true;
							$scope.fetch();
						});
					};

					$scope.clearStatus = function () {
						$scope.saveChangesStatus = undefined;
					};

					$scope.saveChanges = function() {

						if($scope.settings.secretKey && !$scope.submitEncryptionKey) {
							$scope.status = {
								message: lang.t("You must provide the encryption key"),
								type: 'danger',
								closeable: true,
								ttl: 10000
							};
							return;
						}

						$scope.encryptionKey = '';

						var apiParams = Object.assign({}, $scope.settings);
						delete apiParams.userAgreement;
						delete apiParams.privacyPolicy;

						for(var i in apiParams) if(typeof(apiParams[i]) === "boolean") apiParams[i] = apiParams[i] ? 1 : 0;

						if($scope.submitEncryptionKey) apiParams.secretKey = $scope.submitEncryptionKey;
						$scope.savingChanges = true;
						$scope.status = undefined;

						jetapi.manageGDPR(apiParams, function(response) {
							$scope.savingChanges = false;
							$scope.submitEncryptionKey = '';

							if(!response.status) {
								$scope.status = {
									message: lang.t(response.errors[0]),
									type: 'danger',
									closeable: true,
									ttl: 10000
								};
								return;
							}

							$scope.status = {
								message: lang.t("GDPR Settings saved successfully"),
								type: 'success',
								closeable: true,
								ttl: 10000
							};

							if(response.data.secret_key) {
								$scope.settings.secretKey = true;
								$scope.encryptionKey = response.data.secret_key;
							}

						});
					};

					$scope.fetch = function () {
						if($scope.loading) return;
						$scope.loading = true;

						jetapi.getGDPR({}, function(response) {

							$scope.loading = false;

							if(!response.status) {
								$scope.status = {
									message: lang.t(response.errors[0]),
									type: 'danger',
									closeable: true,
									ttl: 10000
								};
								return;
							}
							$scope.settings = response.data;
							$scope.showTerms = !$scope.settings.termsAgreed;
						});

					};

                    $timeout($scope.fetch);
                }]
        );

    }
);